import PropTypes from 'prop-types';
import { cloneDeep } from 'lodash';
import React, { PureComponent } from 'react';
import { Form, Table, Input, InputNumber, Button, Select, Switch, Popconfirm } from 'antd';
import styles from './index.less';
import config from '../../utils/config';

const FormItem = Form.Item;
const InputGroup = Input.Group;
const ButtonGroup = Button.Group;
const { Option } = Select;

const EditableCell = ({ editable, value, onChange }) => (
  <div>
    {editable
      ? <Input style={{ margin: '-5px 0' }} value={value} onChange={e => onChange(e.target.value)} />
      : value
    }
  </div>
);


class EditableTable extends PureComponent {
  static defaultProps = {
    dataSource: [],  //表格的数据源
    merchants: [],   //渠道方信息
    rowKeyName: '',  //表格行唯一key
  };
  static propTypes = {
    dataSource: PropTypes.array,
    rowKeyName: PropTypes.string,
    merchants: PropTypes.array,
  };
  state = {
    data: [],
    cacheData: [],   //缓存数据
  };

  componentWillReceiveProps(nextProps) {
    const { dataSource } = nextProps;
    const cacheData = dataSource.map(item => ({...item}));
    this.setState({
      data: dataSource,
      cacheData: cacheData,
    });
  }

  renderColumns(text, record, column) {
    return (
      <EditableCell
        editable={record.editable}
        value={text}
        onChange={value => this.handleChange(value, record.key, column)}
      />
    );
  }

  handleChange = (value, key, column) => {
    const newData = [...this.state.data];
    const target = newData.filter(item => key === item.key)[0];
    if (target) {
      target[column] = value;
      this.setState({ data: newData });
    }
  }

  handleSave = (key) => {
    const newData = [...this.state.data];
    const target = newData.filter(item => key === item.key)[0];
    if (target) {
      delete target.editable;
      this.setState({
        data: newData,
        cacheData: newData.map(item => ({...item})),
      });
    }
  }

  handleCancel = (key) => {
    const newData = [...this.state.data];
    const target = newData.filter(item => key === item.key)[0];
    if (target) {
      Object.assign(target, this.state.cacheData.filter(item => key === item.key)[0]);
      delete target.editable;
      this.setState({
        data: newData,
      });
    }
  }

  handleEdit = (key) => {
    const newData = [...this.state.data];
    const target = newData.filter(item => key === item.key)[0];
    if (target) {
      target.editable = true;
      this.setState({
        data: newData,
      })
    }
  }

  render() {
    const { data } = this.state;
    const { rowKeyName, merchants } = this.props;
    const columns = [{
      title: '渠道方平台',
      dataIndex: 'merchantId',
      width: '15%',
      render: (text, record) => (
        <Select
          style={{ width: '100%' }}
          disabled={!record.editable}
          defaultValue={record.merchantId}
        >
          {
            merchants.map(item =>
              <Option key={item.merchantId} value={item.merchantId}>{item.merchantName}</Option>
            )
          }
        </Select>
      ),
    },{
      title: '价格类型',
      dataIndex: 'name',
      width: '15%',
      render: (text, record) => (
        <Select
          style={{ width: '100%' }}
          disabled={!record.editable}
        >
          {
            config.PRICE_TYPE.map(item =>
              <Option key={item.value} value={item.value}>{item.name}</Option>
            )
          }
        </Select>
      ),
    },{
      title: '供应商价格',
      dataIndex: 'cpPrice',
      width: '15%',
      render: (text, record) => this.renderColumns(text, record, 'cpPrice'),
    },{
      title: '渠道方价格',
      dataIndex: 'merchantPrice',
      width: '15%',
      render: (text, record) => this.renderColumns(text, record, 'merchantPrice'),
    },{
      title: '终端用户价格',
      dataIndex: 'terminalPrice',
      width: '15%',
      render: (text, record) => this.renderColumns(text, record, 'terminalPrice'),
    },{
      title: '上架状态',
      dataIndex: 'status',
      width: '15%',
      render: (text, record) => (
        <Switch disabled={!record.editable} checkedChildren="上架" unCheckedChildren="下架" />
      ),
    },{
      title: '操作',
      dataIndex: 'operation',
      width: '20%',
      render: (text, record) => {
        const { editable } = record;
        return (
          <div className={styles.editableRowOperations}>
            {editable ?
              <span>
                <a onClick={() => this.handleSave(record.key)}>保存</a>
                <Popconfirm title="确定取消吗?" onConfirm={() => this.handleCancel(record.key)}>
                  <a>取消</a>
                </Popconfirm>
              </span>
              :
              <span>
                <a onClick={() => this.handleEdit(record.key)}>编辑</a>
                <Popconfirm title="确定删除吗?" onConfirm={() => this.handleCancel(record.key)}>
                  <a>删除</a>
                </Popconfirm>
              </span>
            }
          </div>
        );
      }
    }];

    return (
      <div>
        <Button type="primary" className={styles.editableAddBtn}>添加</Button>
        <Table
          bordered
          columns={columns}
          dataSource={data}
          rowKey={rowKeyName}
        />
      </div>
    );
  }
}

export default EditableTable;
