import PropTypes from 'prop-types';
import { cloneDeep } from 'lodash';
import React, { PureComponent } from 'react';
import { Form, Table, Modal, Input, Button, Row, Col, Select } from 'antd';
import styles from './index.less';

const FormItem = Form.Item;
const InputGroup = Input.Group;
const ButtonGroup = Button.Group;
const { Option } = Select;

class ModalSelectTable extends PureComponent {
  static defaultProps = {
    modalTitle: 'SelectTable', //模态弹框标题
    dataSource: [],            //待选列表数据源
    selectedDataSource: [],    //已选择数据源
    baseColumnSet: [],         //表格通用列配置
    selectMode: 'multiple',    //选择模式(单选: single, 多选: multiple)
    selectSort: false,         //已选列表是否支持排序
    children: {},              //动态加载的子组件(这里指头部的搜索组件)
    rowKeyName: 'id',          //数据表行的唯一key取自哪个字段
    pagination: {},            //表格的分页配置
    loading: false,            //待选列表数据加载状态
    onChange: () => {},        //翻页回调函数
    onSubmit: () => {},        //点击确定按钮时回调函数
  };
  static propTypes = {
    dataSource: PropTypes.array,
    selectedDataSource: PropTypes.array,
    modalTitle: PropTypes.string,
    baseColumnSet: PropTypes.array,
    selectMode: PropTypes.string,
    selectSort: PropTypes.bool,
    children: PropTypes.object,
    rowKeyName: PropTypes.string,
    pagination: PropTypes.object,
    loading: PropTypes.bool,
    onChange: PropTypes.func,
    onSubmit: PropTypes.func,
  };
  state = {
    modalShow: false,            //模态框的显示与隐藏
    tableTabLeft: true,          //待选已选tab切换
    selectedDataStorage: [],     //已选择数据容器
  };

  //显示模态选择组件,由父组件通过refs调用触发
  showModal = () => {
    const { selectedDataSource } = this.props;
    const { selectedDataStorage } = this.state;
    let tmpArr = [];
    selectedDataSource.map(item => tmpArr.push(item));
    this.setState({
      modalShow: true,
      selectedDataStorage: tmpArr,
    });
  }

  //模态框关闭时，状态重置
  closeModal = () => {
    this.setState({
      modalShow: false,
      tableTabLeft: true,
      selectedDataStorage: [],
    });
  }

  //点击提交按钮时触发
  handleModalOk = () => {
    const { selectMode } = this.props;
    const { selectedDataStorage } = this.state;
    this.props.onSubmit(selectedDataStorage);
    this.closeModal();
  }

  //点击取消或关闭按钮时触发
  handleModalCancel = () => {
    this.closeModal();
  }

  //多选模式，切换待选已选tab逻辑
  handleTableTabChange = () => {
    const { tableTabLeft } = this.state;
    this.setState({
      tableTabLeft: !tableTabLeft,
    });
  }

  //待选数据表的页码改变时触发onChange回调函数
  handleModalTableChange = (pagination) => {
    this.props.onChange(pagination);
  }

  handleToBeSelectedRowAdd = (record) => {
    const { selectMode } = this.props;
    const { selectedDataStorage } = this.state;
    const tmp = cloneDeep(selectedDataStorage);

    if (selectMode == 'multiple') {
      tmp.push(record);
      this.setState({
        selectedDataStorage: tmp,
      });
    }
    else if (selectMode == 'single') {
      tmp[0] = record;
      this.setState({
        selectedDataStorage: tmp,
      }, () => this.handleModalOk());
    }
    else {
      console.log('模式选择错误');
    }
  }

  handleSelectedRowDel = (record) => {
    const { selectedDataStorage } = this.state;
    const { rowKeyName } = this.props;
    const list = selectedDataStorage.filter(item => item[rowKeyName] !== record[rowKeyName]);
    this.setState({
      selectedDataStorage: list,
    });
  }

  handleSelectedRowSort = (record, sort_up) => {
    const { selectedDataStorage } = this.state;
    const { rowKeyName } = this.props;

    const arr = cloneDeep(selectedDataStorage);

    const index = arr.findIndex(item => item[rowKeyName] == record[rowKeyName]);

    if (sort_up)
    {
      //第一个元素或者未找到元素不做操作
      if (!index || -1 === index) return;
      //与前一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index - 1, 1, arr[index]));
    }
    else
    {
      //最后一个元素或者未找到元素不做操作
      if (index + 1 === arr.length || -1 === index) return;
      //与后一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index + 1, 1, arr[index]));
    }

    this.setState({
      selectedDataStorage: arr,
    });
  }

  render() {
    const {
      dataSource, baseColumnSet, modalTitle, children, loading,
      selectMode, selectSort, rowKeyName, pagination,
    } = this.props;
    const { modalShow, tableTabLeft, selectedDataStorage } = this.state;

    //给已选列表增加排序序号字段
    selectedDataStorage.map((item, index) => item['sortId'] = index + 1);

    const sortNumberColumn = {
      title: '序号',
      dataIndex: 'sortId',
      width: 50,
      fixed: 'left',
    };
    const addColumn = {
      title: '添加',
      render: (text, record) => (
        <Button
          disabled={selectedDataStorage.filter(item => item[rowKeyName] == record[rowKeyName]).length}
          size="small"
          type="dashed"
          icon="plus"
          onClick={() => this.handleToBeSelectedRowAdd(record)}
        >
        </Button>
      ),
      width: 50,
    };
    const sortColumn = {
      title: '排序',
      render: (text, record) => (
        <ButtonGroup>
          <Button size="small" icon="arrow-up" onClick={() => this.handleSelectedRowSort(record, true)}></Button>
          <Button size="small" icon="arrow-down" onClick={() => this.handleSelectedRowSort(record, false)}></Button>
        </ButtonGroup>
      ),
      width: 80,
    };
    const delColumn = {
      title: '删除',
      render: (text, record) => (
        <Button size="small" type="danger" icon="delete" onClick={() => this.handleSelectedRowDel(record)}></Button>
      ),
      width: 50,
    };

    const toBeSelectedColumns = [];
    const selectedColumns = [];
    baseColumnSet.map(item => {
      toBeSelectedColumns.push(item);
      selectedColumns.push(item);
    });

    toBeSelectedColumns.push(addColumn);

    if (selectSort) {
      selectedColumns.unshift(sortNumberColumn);
    }

    selectedColumns.push(sortColumn);
    selectedColumns.push(delColumn);

    return (
      <Modal
        title={modalTitle}
        visible={modalShow}
        maskClosable={false}
        onOk={this.handleModalOk}
        onCancel={this.handleModalCancel}
      >
        <div className={styles.search}>{children}</div>
        <div className={styles.tableTab} style={selectMode == 'multiple' ? null : { display: 'none' }}>
          <ButtonGroup>
            <Button onClick={this.handleTableTabChange} type="primary" ghost={tableTabLeft ? false : true}>
              待选
            </Button>
            <Button onClick={this.handleTableTabChange} type="primary" ghost={tableTabLeft ? true : false}>
              {`已选[${selectedDataStorage.length}]`}
            </Button>
          </ButtonGroup>
        </div>
        <div className={styles.table}>
          <Table
            size="middle"
            bordered={true}
            columns={tableTabLeft ? toBeSelectedColumns : selectedColumns}
            dataSource={tableTabLeft ? dataSource : selectedDataStorage}
            rowKey={rowKeyName}
            pagination={tableTabLeft ? pagination : false}
            loading={loading}
            onChange={this.handleModalTableChange}
            scroll={{ y: 200 }}
          >
          </Table>
        </div>
      </Modal>
    );
  }
}

export default ModalSelectTable;
