import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { Link, routerRedux } from 'dva/router';
import {
  Popconfirm, Progress, notification, Badge, message, Card,
  Icon, Button, Modal, Input, Select, Form, Row, Col
} from 'antd';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import StandardTable from '../../components/StandardTable';
import styles from './ItemList.less';
import config from '../../utils/config';
import Logger from '../../utils/logger';

const { Option } = Select;
const FormItem = Form.Item;
const ButtonGroup = Button.Group;

const logger = Logger.getLogger('ItemList');
const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');

@connect(state => ({
  item: state.item,
  cp: state.cp,
}))
@Form.create()
export default class ItemList extends PureComponent {
  state = {
    selectedRows: [],   //记录选中的行
    formValues: {},     //记录搜索表单值
  };

  _actionControlCenter(type, payload = {}) {
    const { dispatch } = this.props;
    //对空字段进行过滤
    Object.keys(payload).map(key => {payload[key] ? null : delete payload[key]});
    dispatch({ type, payload });
  }

  componentDidMount() {
    this._actionControlCenter('item/getItemList');
    this._actionControlCenter('cp/getCPList');
  }

  handleSearch = (e) => {
    e.preventDefault();
    const { form } = this.props;
    form.validateFields((err, fieldsValue) => {
      if (err) return;
      const values = {
        ...fieldsValue,
      };
      this.setState({
        formValues: {...fieldsValue},
      });
      this._actionControlCenter('item/getItemList', values);
    });
  }

  handleReset = (e) => {
    e.preventDefault();
    const { form } = this.props;
    form.resetFields();
    this.setState({
      formValues: {},
    });
    this._actionControlCenter('item/getItemList');
  }

  handleSelectRows = (rows) => {
    this.setState({
      selectedRows: rows,
    });
  }

  handleItemAdd = () => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/goods/item/add'));
  }

  handleItemEdit = (record) => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push({
      pathname: '/goods/item/edit',
      state: { code: record.code },       //进入编辑页面传递code参数
    }));
  }

  handleItemDel = (record) => {
    this._actionControlCenter('item/delItemItem', { code: record.code });
    this._actionControlCenter('item/getItemList');
  }

  //配置分页器
  handleStandardTableChange = (pagination, filterArgs, sorter) => {
    const { formValues } = this.state;
    const filters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = {...obj};
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      ...formValues,
      ...filters,
    };
    this._actionControlCenter('item/getItemList', params);
  }

  render() {
    const { selectedRows } = this.state;
    const { dispatch, item, cp } = this.props;
    const { getFieldDecorator } = this.props.form;

    const columns = [{
      title: '商品编号',
      dataIndex: 'code',
    },{
      title: '商品名称',
      dataIndex: 'name',
    },{
      title: '内容供应商',
      dataIndex: 'cpName',
      filters: cp.cpList.map(item => ({
        text: item.cpName,
        value: item.cpId,
      })),
    },{
      title: '标签名称',
      dataIndex: 'tagList',
      render: (text, record) => (
        <p>{record.tagList.map(item => item.name).join(',')}</p>
      ),
    },{
      title: '使用状态',
      dataIndex: 'status',
      filters: config.ITEM_STATE.map(item => ({
        text: item.name,
        value: item.value,
      })),
      render: (text, record) => (
        <span>
          {(config.ITEM_STATE.filter(item => item.value === record.status)[0] || { name: '未知' }).name}
        </span>
      ),
    },{
      title: '修改时间',
      dataIndex: 'gmtModified',
    },{
      title: '操作类型',
      render: (record) => (
        <span>
          <Button size="small" shape="circle" icon="edit" onClick={() => this.handleItemEdit(record)}></Button>
          <span className={styles.splitLine} />
          <Popconfirm
            placement="top"
            okText="删除"
            cancelText="取消"
            title='确定删除?'
            onConfirm={() => this.handleItemDel(record)}
          >
            <Button size="small" shape="circle" icon="delete"></Button>
          </Popconfirm>
        </span>
      ),
    }];

    const standardTableProps = {
      dataSource: item.data.list,
      pagination: item.data.pagination,
      loading: item.loading,
      columns: columns,
      rowKeyName: 'id',
      selectedRows: selectedRows,
    };

    const pageHeaderSearch = (
      <Form layout="inline" onSubmit={this.handleSearch}>
        <FormItem label="商品编号">
          {getFieldDecorator('code')(
            <Input placeholder="请输入商品编号" />
          )}
        </FormItem>
        <FormItem label="商品名称">
          {getFieldDecorator('name')(
            <Input placeholder="请输入商品名称" />
          )}
        </FormItem>
        <FormItem>
          <ButtonGroup>
            <Button icon="search" type="primary" htmlType="submit">搜索</Button>
            <Button icon="reload" type="danger" onClick={this.handleReset}>重置</Button>
          </ButtonGroup>
        </FormItem>
      </Form>
    );

    return (
      <PageHeaderLayout
        content={pageHeaderSearch}
      >
        <div className={styles.content}>
          <div className={styles.newButton}>
            <Button type="primary" icon="plus" onClick={() => {this.handleItemAdd()}}>添加</Button>
          </div>
          <StandardTable
            {...standardTableProps}
            onChange={this.handleStandardTableChange}
            onSelectRow={this.handleSelectRows}
          />
        </div>
      </PageHeaderLayout>
    );
  }
}
