import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import React, { PureComponent } from 'react';
import { Form, Input, Select, Card, Button, Table } from 'antd';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import ModalSelectTable from '../../components/ModalSelectTable';
import EditableTable from '../../components/EditableTable';
import styles from './SupportSave.less';
import { getParentPath } from '../../utils/utils';
import config from '../../utils/config';
import Logger from '../../utils/logger';

const FormItem = Form.Item;
const InputGroup = Input.Group;
const { TextArea } = Input;
const { Option } = Select;

const { ROUTER_MAP, ACTION_MAP } = config;
const logger = Logger.getLogger('SupportSave');

@connect(state => ({
  cp: state.cp,
  tag: state.tag,
  item: state.item,
  support: state.support,
  merchant: state.merchant,
}))
@Form.create()
export default class SupportSave extends PureComponent {
  state = {
    endPoint: '',          //操作类型，add或edit
    parentPath: '',        //路径前缀
    searchSelect: 'name',  //模态框的搜索字段
    searchInput: '',       //模态框的搜索字段内容
  };

  _actionControlCenter = (type, payload = {}) => {
    const { dispatch } = this.props;
    //对空字段进行过滤
    Object.keys(payload).map(key => {payload[key] ? null : delete payload[key]});
    //触发modal中action
    dispatch({ type, payload });
  }

  _routerControlCenter = (router = '/', payload = null) => {
    const { dispatch } = this.props;
    if (payload) {
      dispatch(routerRedux.push({ pathname: router, state: payload }));
    }
    else {
      dispatch(routerRedux.push(router));
    }
  }

  componentWillMount() {
    const { location } = this.props;
    const { endPoint, parentPath } = getParentPath(location);
    if (endPoint === 'edit' && !location.state) {
      this._routerControlCenter(parentPath);
    }
    else {
      this.setState({ endPoint, parentPath });
    }
  }

  componentDidMount() {
    const { location } = this.props;
    const { endPoint, parentPath } = this.state;

    if (endPoint !== 'add' && endPoint !== 'edit') {
      this._routerControlCenter(parentPath);
    }

    //如果是编辑选项，则请求编辑项的相关周边信息/相关商品信息
    if (endPoint === 'edit') {
      this._actionControlCenter(ACTION_MAP.supportItem, location.state);
      this._actionControlCenter(ACTION_MAP.itemItem, {} ); //TODO
    }
    this._actionControlCenter(ACTION_MAP.supportList);
    this._actionControlCenter(ACTION_MAP.merchantList);
    this._actionControlCenter(ACTION_MAP.tagList);
    this._actionControlCenter(ACTION_MAP.cpList);
  }

  componentWillUnmount(){
    this._actionControlCenter(ACTION_MAP.supportClear);
    this.setState({ endPoint: '', parentPath: '' });
  }

  //点击编辑周边按钮，出现模态框
  handleEditSupportBtnClick = () => {
    this.refs.getSupportModalSelectTable.showModal();
  }

  //模态框中的待选列表翻页动作
  handleToBeSelectedTableChange = (pagination) => {
    const { searchSelect, searchInput } = this.state;
    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
    };
    params[searchSelect] = searchInput;

    this._actionControlCenter(ACTION_MAP.supportList, params);
  }

  //相关周边列表点击搜索按钮进行查询
  handleSearchBtnClick = () => {
    const { searchSelect, searchInput } = this.state;
    let searchContent = {};
    searchContent[searchSelect] = searchInput;
    this._actionControlCenter(ACTION_MAP.supportList, {...searchContent});
  }

  //模态框中的重置按钮处理动作
  handleSearchResetClick = () => {
    this.setState({ searchSelect: 'name', searchInput: '' });
    this._actionControlCenter(ACTION_MAP.supportList);
  }

  //模态框中的搜索字段改变触发
  handleSearchSelectChange = (value) => {
    this.setState({ searchSelect: value });
  }

  //模态框中的搜索内容输入框变化触发
  handleSearchInputChange = (e) => {
    this.setState({ searchInput: e.target.value });
  }

  //页面取消按钮触发
  handlePageCancel = () => {
    this._routerControlCenter(ROUTER_MAP.supportList);
  }

  //页面提交按钮触发
  handlePageSubmit = (e) => {
    e.preventDefault();

    const { endPoint } = this.state;
    const { form, dispatch, support: { supportItem } } = this.props;

    form.validateFields((err, fieldsValue) => {
      if (err) return;
      const values = { ...supportItem, ...fieldsValue};
      if (endPoint === 'edit')
      {
        this._actionControlCenter(ACTION_MAP.supportItemUpdate, values);
      }
      else if (endPoint === 'add')
      {
        this._actionControlCenter(ACTION_MAP.supportItemAdd, values);
      }
      else
      {
        message.error('未知操作类型!', 5);  //不可能走到这里
      }

      form.resetFields();
      this._routerControlCenter(ROUTER_MAP.supportList);
    });
  }

  //模态框提交按钮触发
  handleModalSubmit = (selectedDataStorage) => {
    const { support: { supportItem } } = this.props;
    this._actionControlCenter(
      ACTION_MAP.supportSaveItemData,
      {...supportItem, supportArr: selectedDataStorage}
    );
  }

  render() {
    const { cp, tag, item, merchant, support: { supportItem, loading, data: { list, pagination } } } = this.props;
    const { searchContent } = this.state;
    const { getFieldDecorator } = this.props.form;

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 },
        md: { span: 6 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 12 },
        md: { span: 13 },
      },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    const baseColumnSet = [{
      title: '周边编号',
      dataIndex: 'code',
      width: 150,
    },{
      title: '周边名称',
      dataIndex: 'name',
      width: 150,
    }];

    return(
      <PageHeaderLayout>
        <div className={styles.content}>
          <Card title="基本信息" bordered={true} style={{ marginBottom: 24 }}>
            <Form onSubmit={this.handlePageSubmit}>
              <FormItem
                label="周边编号"
                {...formItemLayout}
              >
                {getFieldDecorator('code', {
                    rules: [{ required: true, type: 'string', message: '周边编号必须填写!' }],
                    initialValue: supportItem.code,
                })(<Input placeholder="请输入周边编号" />)}
              </FormItem>
              <FormItem
                label="周边名称"
                {...formItemLayout}
              >
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: '周边名称必须填写!' }],
                  initialValue: supportItem.name,
                })(<Input placeholder="请输入周边名称" />)}
              </FormItem>
              <FormItem
                label="周边描述"
                {...formItemLayout}
              >
                {getFieldDecorator('digest', {
                  initialValue: supportItem.digest,
                })(<TextArea rows={4} placeholder="请添加周边描述" />)}
              </FormItem>
              <FormItem
                label="状态"
                {...formItemLayout}
              >
                {getFieldDecorator('status', {
                  initialValue: supportItem.status,
                })(
                  <Select>
                    {
                      config.SUPPORT_STATE.map(item =>
                        <Option key={item.value} value={item.value}>{item.name}</Option>
                      )
                    }
                  </Select>
                )}
              </FormItem>
              <FormItem
                label="标签"
                {...formItemLayout}
              >
                {getFieldDecorator('tags', {
                    initialValue: supportItem.tags.map(item => item.tagId),
                })(
                  <Select
                    mode="multiple"
                    placeholder="请选择标签"
                    notFoundContent="标签数据加载失败"
                  >
                    {tag.data.list.map(item =>
                      <Option key={item.tagId} value={item.tagId}>{item.tagName}</Option>
                    )}
                  </Select>
                )}
              </FormItem>
              <FormItem
                label="内容提供商"
                {...formItemLayout}
              >
                {getFieldDecorator('cpId', {
                    initialValue: supportItem.cpId,
                })(
                  <Select
                    placeholder="请选择内容提供商"
                    notFoundContent="内容提供商信息加载失败"
                  >
                    {cp.cpList.map(item =>
                      <Option key={item.cpId} value={item.cpId}>{item.cpName}</Option>
                    )}
                  </Select>
                )}
              </FormItem>
              <FormItem
                label="周边相关"
                {...formItemLayout}
              >
                <Button
                  icon="edit"
                  type="primary"
                  onClick={() => {this.handleEditSupportBtnClick()}}
                >
                  编辑周边
                </Button>
              </FormItem>
              <FormItem
                label="相关周边"
                {...formItemLayout}
              >
                <Table
                  size="middle"
                  bordered={true}
                  dataSource={supportItem.supportArr}
                  columns={baseColumnSet}
                  rowKey={record => record.code}
                  pagination={{ pageSize: 20 }}
                  scroll={{ y: 200 }}
                />
              </FormItem>
              <ModalSelectTable
                ref="getSupportModalSelectTable"
                modalTitle="相关周边"
                dataSource={list}
                baseColumnSet={baseColumnSet}
                selectedDataSource={supportItem.supportArr}
                selectSort={true}
                selectMode='multiple'
                pagination={pagination}
                rowKeyName="code"
                loading={loading}
                onChange={this.handleToBeSelectedTableChange}
                onSubmit={this.handleModalSubmit}
              >
                <div>
                  <InputGroup compact>
                    <Select
                      onChange={this.handleSearchSelectChange}
                      defaultValue="name"
                      style={{ width: '27%' }}
                    >
                      <Option value="name">周边配套名称</Option>
                      <Option value="code">周边配套编号</Option>
                    </Select>
                    <Input
                      onChange={this.handleSearchInputChange}
                      onPressEnter={this.handleSearchBtnClick}
                      placeholder="请输入搜索内容"
                      style={{ width: '45%' }}
                    />
                    <Button
                      type="primary"
                      style={{ width: '14%' }}
                      onClick={this.handleSearchBtnClick}
                    >搜索
                    </Button>
                    <Button
                      style={{ width: '14%' }}
                      onClick={this.handleSearchResetClick}
                    >重置
                    </Button>
                  </InputGroup>
                </div>
              </ModalSelectTable>
            </Form>
          </Card>
          <Card title="价格信息" style={{ marginBottom: 24 }} bordered={true}>
            <EditableTable
              dataSource={item.itemItem.priceList}
              merchants={merchant.merchantList}
              rowKeyName={'id'}
            />
          </Card>
          <Card bordered={false}>
            <Button onClick={this.handlePageCancel}>取消</Button>
            <Button style={{ marginLeft: 8 }} type="primary" onClick={(e) => this.handlePageSubmit(e)}>提交</Button>
          </Card>
        </div>
      </PageHeaderLayout>
    );
  }
}
