import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import { connect } from 'dva';
import { Popconfirm, Button, Input, Select, Form } from 'antd';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import StandardTable from '../../components/StandardTable';
import styles from './TagList.less';
import config from '../../utils/config';
import Logger from '../../utils/logger';

const logger = Logger.getLogger('TagRoute');

const { Option } = Select;
const FormItem = Form.Item;
const ButtonGroup = Button.Group;

const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');

@connect(state => ({
  tag: state.tag,
  merchant: state.merchant,
}))
@Form.create()
export default class TagList extends PureComponent {
  state = {
    modalVisible: false,
    selectedRows: [],
    formValues: {},
  };

  _actionControlCenter(type, payload = {}) {
    const { dispatch } = this.props;
    //对空字段进行过滤
    Object.keys(payload).map(key => {payload[key] ? null : delete payload[key]});
    dispatch({ type, payload });
  }

  componentDidMount() {
    this._actionControlCenter('merchant/getAllMerchants');
    this._actionControlCenter('tag/getTagList');
  }

  handleSearch = (e) => {
    e.preventDefault();
    const { dispatch, form } = this.props;
    form.validateFields((err, fieldsValue) => {
      if (err) return;
      const values = {
        ...fieldsValue,
      }
      this._actionControlCenter('tag/getTagList', values);
    });
  }

  handleReset = (e) => {
    e.preventDefault();
    const { dispatch, form } = this.props;
    form.resetFields();
    this._actionControlCenter('tag/getTagList');
  }

  handleSelectRows = (rows) => {
    this.setState({
      selectedRows: rows,
    });
  }

  handleItemAdd = () => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/product/tag/add'));
  }

  handleItemEdit = (record) => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/product/tag/edit', { tagId: record.tagId }));
  }

  handleItemDel = (record) => {
    this._actionControlCenter('tag/delTagItem', { tagId: record.tagId });
    this._actionControlCenter('tag/getTagList');
  }

  //配置分页器
  handleStandardTableChange = (pagination, filterArgs, sorter) => {
    const { dispatch } = this.props;
    const { formValues } = this.state;

    const filters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = {...obj};
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      ...formValues,
      ...filters,
    };

    this._actionControlCenter('tag/getTagList', params);
  }

  render() {
    const { selectedRows, modalVisible } = this.state;
    const { dispatch, tag, merchant } = this.props;
    const { getFieldDecorator } = this.props.form;

    const columns = [{
      title: '标签名称',
      dataIndex: 'tagName',
    },{
      title: '标签类型',
      dataIndex: 'tagType',
      filters: config.TAG_TYPE.map((item) => ({
        text: item.name,
        value: item.value,
      }))
    },{
      title: '渠道平台',
      dataIndex: 'merchantName',
      filters: merchant.merchantList.map((item) => ({
        text: item.merchantName,
        value: item.merchantId,
      }))
    },{
      title: '修改时间',
      dataIndex: 'gmtModified',
    },{
      title: '操作类型',
      render: (record) => (
        <p>
          <Button size="small" shape="circle" icon="edit" onClick={() => this.handleItemEdit(record)}></Button>
          <span className={styles.splitLine} />
          <Popconfirm
            placement="top"
            okText="删除"
            cancelText="取消"
            title='确定删除?'
            onConfirm={() => this.handleItemDel(record)}
          >
            <Button size="small" shape="circle" icon="delete"></Button>
          </Popconfirm>
        </p>
      ),
    }];

    const standardTableProps = {
      dataSource: tag.data.list,
      pagination: tag.data.pagination,
      loading: tag.loading,
      columns: columns,
      rowKeyName: 'tagId',
      selectedRows: selectedRows,
    };

    const pageHeaderSearch = (
      <Form
        layout="inline"
        onSubmit={this.handleSearch}
        ref="searchForm"
      >
        <FormItem label="标签名称">
          {getFieldDecorator('tagName')(
            <Input placeholder="请输入标签名称" />
          )}
        </FormItem>
        <FormItem label="标签类型">
          {getFieldDecorator('tagType')(
            <Select style={{ width: 140 }}>
              {
                config.TAG_TYPE.map(item =>
                  <Option key={item.value} value={item.value}>{item.name}</Option>
                )
              }
            </Select>
          )}
        </FormItem>
        <FormItem label="渠道平台">
          {getFieldDecorator('merchantId')(
            <Select style={{ width: 140 }}>
              {
                merchant.merchantList.map(item =>
                  <Option key={item.merchantId} value={item.merchantId}>{item.merchantName}</Option>
                )
              }
            </Select>
          )}
        </FormItem>
        <FormItem>
          <ButtonGroup>
            <Button icon="search" type="primary" htmlType="submit">搜索</Button>
            <Button icon="reload" type="danger" onClick={this.handleReset}>重置</Button>
          </ButtonGroup>
        </FormItem>
      </Form>
    );

    return (
      <PageHeaderLayout
        content={pageHeaderSearch}
      >
        <div className={styles.content}>
          <div className={styles.newButton}>
            <Button type="primary" icon="plus" onClick={() => {this.handleItemAdd()}}>添加</Button>
          </div>
          <StandardTable
            {...standardTableProps}
            onChange={this.handleStandardTableChange}
            onSelectRow={this.handleSelectRows}
          />
        </div>
      </PageHeaderLayout>
    );
  }
}
